from PIL import Image
import io
def autoorient(image):
    """Accepts a PIL image item as input, returns a PIL image item as output."""
    # get exif data from image
    try:
        exif = image._getexif()
    except AttributeError:
        exif = None

    # if image has no exif data, return it unmodified
    if exif is None:
        return image

    # define exif orientation values and corresponding transformations
    ORIENTATIONS = {
        3: (Image.ROTATE_180,),
        6: (Image.ROTATE_270,),
        8: (Image.ROTATE_90,),
        2: (Image.FLIP_LEFT_RIGHT,),
        4: (Image.FLIP_TOP_BOTTOM, Image.ROTATE_180),
        5: (Image.FLIP_LEFT_RIGHT, Image.ROTATE_270),
        7: (Image.FLIP_LEFT_RIGHT, Image.ROTATE_90),
    }
    
    orientation = exif.get(274)
    if orientation not in ORIENTATIONS:
        return image

    for transform in ORIENTATIONS[orientation]:
        image = image.transpose(transform)


    return image


def resize_and_autoorient(file, height,width):
    """Accepts a file bytes object and returns a file bytes object
    Resizes an image based on specifications in the config."""
    f = Image.open(file)
    f = autoorient(f)
    f=f.resize((int(height),int(width)))
    # Create a bytes object to send in response
    img_file = io.BytesIO()
    f.save(img_file, format='JPEG')
    img_file.seek(0)
    return img_file


import zipfile
import io
def add_image_to_word_file(word_file, icon_file, imagepath):
    """Adds an icon to a word file by first unzipping it, replacing an icon png file with a bytes object. returns a compressed BytesIO. No further processing is necessary to use as a document.

    Args:
        word_file (Bytes): Word file as a Bytes - object.
        icon_file (Bytes): Image object to be passed to the placeholder.
        image_path (String): Path to the image inside the Word file. To find this, change the file ending to .zip and locate the filename in /word/media

    Returns:
        BytesIO: BytesIO object to be opened or saved for word-document processing.
    """
    with zipfile.ZipFile(io.BytesIO(word_file), mode='r') as zf:

        files = {x: zf.read(x) for x in zf.namelist()}
        files[imagepath] = icon_file

    # Create a new in-memory ZipFile object
    new_word_file = io.BytesIO()
    with zipfile.ZipFile(new_word_file, mode='w') as zkf:
        for name, file in files.items():
            # Add each file to the new zipfile
            zkf.writestr(name, file)

    # Reset the output_zip's file pointer to the beginning
    new_word_file.seek(0)
    return new_word_file
def compress_doc(word_file, icon_file, imagepath):
    """Adds an icon to a word file by first unzipping it, replacing an icon png file with a bytes object. returns a compressed BytesIO. No further processing is necessary to use as a document.

    Args:
        word_file (Bytes): Word file as a Bytes - object.
        icon_file (Bytes): Image object to be passed to the placeholder.
        image_path (String): Path to the image inside the Word file. To find this, change the file ending to .zip and locate the filename in /word/media

    Returns:
        BytesIO: BytesIO object to be opened or saved for word-document processing.
    """
    with zipfile.ZipFile(io.BytesIO(word_file), mode='r') as zf:

        files = {x: zf.read(x) for x in zf.namelist()}
        files[imagepath] = icon_file

    # Create a new in-memory ZipFile object
    new_word_file = io.BytesIO()
    with zipfile.ZipFile(new_word_file, mode='w', compression=zipfile.ZIP_DEFLATED) as zkf:
        for name, file in files.items():
            # Add each file to the new zipfile
            zkf.writestr(name, file)

    # Reset the output_zip's file pointer to the beginning
    new_word_file.seek(0)
    return new_word_file



import datetime



def get_first_and_last_week_of_month(year, month):
    # Calculate the first day of the ISO year
    jan4 = datetime.date(year, 1, 4)
    
    # Calculate the first day of the first week of the ISO year
    delta = datetime.timedelta(days=7-jan4.weekday())
    first_day_of_first_week = jan4 + delta
    
    # Calculate the date of the first day of the specified month
    first_day_of_month = datetime.date(year, month, 1)
    
    # Calculate the number of days between the first day of the first week and the first day of the month
    days_between = (first_day_of_month - first_day_of_first_week).days
    
    # Calculate the number of weeks between the first day of the first week and the first day of the month
    weeks_between = int(days_between / 7)
    
    # Calculate the first day of the first week of the specified month
    first_day_of_first_week_of_month = first_day_of_first_week + datetime.timedelta(days=weeks_between*7)+datetime.timedelta(days=7)
    
    # Calculate the last day of the last week of the specified month
    last_day_of_month = datetime.date(year, month, 28)
    while last_day_of_month.month == month:
        last_day_of_month += datetime.timedelta(days=1)
    last_day_of_month -= datetime.timedelta(days=7-last_day_of_month.weekday())
    
    return first_day_of_first_week_of_month, last_day_of_month+datetime.timedelta(days=7)-datetime.timedelta(days=1)

def get_month_from_year_week(year, week):
    # Calculate the first day of the ISO year
    jan4 = datetime.date(year, 1, 4)

    # Calculate the first day of the first week of the ISO year
    delta = datetime.timedelta(days=-jan4.weekday())
    first_day_of_first_week = jan4 + delta
    
    # Calculate the date of the first day of the specified week
    first_day_of_week = first_day_of_first_week + datetime.timedelta(weeks=week-1)

    # Extract the month number from the first day of the week
    month = first_day_of_week.month
    
    return month

def get_week_numbers(daterange):
    weeklist = []
    for date in daterange:
        weeklist.append(date.isocalendar().week)
    return list(set(weeklist))
    
def get_month_name_from_number(month):
    months = ["Januari","Februari","Mars","April","Maj","Juni","Juli","Augusti","September","Oktober","November","December"]
    return months[month-1]

def get_month_from_week(year, week_number):
    # Create a datetime object for the given year and week number
    if not year: year="2023"
    dt = datetime.datetime.strptime(f'{year}-W{week_number}-1', '%Y-W%W-%w')

    # Get the month in the ISO format
    month = dt.strftime('%Y-%m')

    return month
def first_day_from_week(year,week):
    # Create a date object for January 4th of the given year, which is always in week 1
    jan_4 = datetime.date(year, 1, 4)

    # Calculate the number of days to add to January 4th to get to the first day of the given week
    days_to_add = (week - 1) * 7 - jan_4.weekday()

    # Calculate the date of the first day of the given week
    first_day = jan_4 + datetime.timedelta(days=days_to_add)

    return first_day


from bs4 import BeautifulSoup
import os
import zipfile
import io

def set_checkbox_value(xlsx_data, checkbox_value, checkbox_location):
    """Takes a Word document as a bytes object as input (from file.read() for example), returns a BytesIO to be opened by various libraries with one checkbox ticked.

    Args:
        xlsx_data (Bytes): Bytes representation of a Word document
        checkbox_value (int): 1 = checked, 0 = unchecked
        checkbox_location (int): which location from 0 to x your checkbox has in the number of checkboxes

    Returns:
        BytesIO: Returns a BytesIO to be used in document libraries
    """
    with zipfile.ZipFile(io.BytesIO(xlsx_data), mode='r') as zf:

        sheet1_data = zf.read('word/document.xml').decode('utf-8')
        print(sheet1_data.index('w:sdt'))
        soup = BeautifulSoup(sheet1_data, 'xml')

        std_elements_with_checkboxes = soup.find_all('w:sdt')
        x =std_elements_with_checkboxes[checkbox_location]
        if "checkbox" in str(x): 
            x.find('checked')['w14:val'] = checkbox_value
            x.find('w:t').string = ["☒" if checkbox_value==1 else "☐"][0]
                
        print(len(std_elements_with_checkboxes))
        
        files = {x: zf.read(x) for x in zf.namelist()}
        modified_sheet1_data = str(soup)
        files['word/document.xml'] = modified_sheet1_data

    # Create a new in-memory ZipFile object
    output_zip = io.BytesIO()
    with zipfile.ZipFile(output_zip, mode='w') as zkf:
        for name, file in files.items():
            # Add each file to the new zipfile
            zkf.writestr(name, file)

    # Reset the output_zip's file pointer to the beginning
    output_zip.seek(0)
    return output_zip



if __name__ == '__main__':
    with open(os.path.join(os.path.dirname(__file__),'Fitness mall cert.docx'),'rb') as fh:
        file=set_checkbox_value(fh.read(), 1, 0)
    with open(os.path.join(os.path.dirname(__file__),'Fitness mall cert2.docx'),'wb') as fh:
        fh.write(file.read())
        



from setuptools import find_packages, setup
setup(
    name='Office_helper_functions',
    packages=find_packages(),
    version='0.1.1',
    description='Helper functions for Microsoft Office',
    author='Andreas Löfkvist',
    author_email='andreasmlofkvist@gmail.com',
    license='MIT',
    setup_requires=['beautifulsoup4', 'pillow',]
)